import {
  AppstoreOutlined,
  CalendarOutlined,
  MessageOutlined,
  QuestionCircleOutlined,
  ScissorOutlined,
  SettingOutlined,
  ShopOutlined,
  TeamOutlined,
  UserOutlined,
  WalletOutlined,
} from "@ant-design/icons";
import { Button, Card, Divider, Flex, Menu, Typography } from "antd";
import { useEffect, useState } from "react";
import { useLocation, useNavigate } from "react-router-dom";
import { BreakpointLgWidth, Constants, isDevelopmentEnv } from "../../utils";
import { useTranslation } from "react-i18next";
import { useSideBarContext } from "../../Contexts/SideBarContext";
import { useStoresContext } from "../../Contexts/StoresContext";
import MyAppLogo from "../MyAppLogo";

export function SideMenuContent({
  setIsSideMenuCollapsed,
  contentFirstRender,
}) {
  const sideBarContext = useSideBarContext();
  const storesContext = useStoresContext();
  const location = useLocation();
  const [selectedKeys, setSelectedKeys] = useState("/");
  const [openKeys, setOpenKeys] = useState([""]);
  const { t } = useTranslation();

  const navigate = useNavigate();

  const getFirstMenuItems = () => {
    let items = [
      {
        label: t("sideMenu.overview"),
        icon: <AppstoreOutlined />,
        key: Constants.ROUTE_PATHS.OVERVIEW,
      },
    ];

    // stores

    let stores = {
      label:
        storesContext.stores.length > 1
          ? t("sideMenu.store.titlePlural")
          : t("sideMenu.store.titleSingular"),
      children: [],
      type: "group",
    };

    storesContext.stores.forEach((store) => {
      let groupStore = {
        label: store.name,
        icon: <ShopOutlined />,
        children: [],
        key: `${Constants.ROUTE_PATHS.STORE.OVERVIEW}/${store.store_id}`,
      };

      if (sideBarContext.permissions.includes("settings")) {
        groupStore.children.push({
          label: t("sideMenu.store.settings"),
          icon: <SettingOutlined />,
          key: `${Constants.ROUTE_PATHS.STORE.OVERVIEW}/${store.store_id}/${Constants.ROUTE_PATHS.STORE.SETTINGS}`,
        });
      }

      if (sideBarContext.permissions.includes("employees")) {
        groupStore.children.push({
          label: t("sideMenu.store.employees"),
          icon: <TeamOutlined />,
          key: `${Constants.ROUTE_PATHS.STORE.OVERVIEW}/${store.store_id}/${Constants.ROUTE_PATHS.STORE.EMPLOYEES}`,
        });
      }

      if (sideBarContext.permissions.includes("services")) {
        groupStore.children.push({
          label: t("sideMenu.store.services"),
          icon: <ScissorOutlined />,
          key: `${Constants.ROUTE_PATHS.STORE.OVERVIEW}/${store.store_id}/${Constants.ROUTE_PATHS.STORE.SERVICES}`,
        });
      }

      if (sideBarContext.permissions.includes("calendar")) {
        groupStore.children.push({
          label: t("sideMenu.store.calendar"),
          icon: <CalendarOutlined />,
          key: `${Constants.ROUTE_PATHS.STORE.OVERVIEW}/${store.store_id}/${Constants.ROUTE_PATHS.STORE.CALENDAR}`,
        });
      }

      stores.children.push(groupStore);
    });

    items.push(stores);

    return items;
  };

  const showPaymentPlanInfoBanner =
    sideBarContext.permissions.includes("paymentPlan") &&
    sideBarContext.paymentPlanStatus === "trialing" &&
    sideBarContext.paymentPlanTrialEnd !== null &&
    sideBarContext.paymentPlanCanceledAt !== null;

  const getSecondMenuItems = () => {
    let items = [];

    if (isDevelopmentEnv()) {
      items.push(
        {
          label: t("sideMenu.support"),
          icon: <QuestionCircleOutlined />,
          key: Constants.ROUTE_PATHS.SUPPORT,
        },
        {
          label: t("sideMenu.feedback"),
          icon: <MessageOutlined />,
          key: Constants.ROUTE_PATHS.FEEDBACK,
        }
      );
    }

    if (!showPaymentPlanInfoBanner) {
      items.push({
        label: t("sideMenu.paymentPlan"),
        icon: <WalletOutlined />,
        key: Constants.ROUTE_PATHS.PAYMENT_PLAN,
      });
    }

    items.push({
      label: sideBarContext.username,
      icon: <UserOutlined />,
      key: Constants.ROUTE_PATHS.USER_PROFILE,
    });

    return items;
  };

  useEffect(() => {
    const pathname = location.pathname;

    setSelectedKeys(pathname);

    let path = pathname.split("/");

    if (path.length > 2) {
      // /store/:storeId/:subPage - open the store menu
      setOpenKeys([`/${path[1]}/${path[2]}`]);
    }

    // auto close sideMenu on mobile
    // this will prevent to auto close sideMenu on first render as the useEffects will be called after the first render
    if (contentFirstRender.current) {
      contentFirstRender.current = false;
      //return;
    } else if (
      /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(
        navigator.userAgent
      ) ||
      document.body.clientWidth < BreakpointLgWidth
    ) {
      setIsSideMenuCollapsed(true);
    }
  }, [location.pathname]);

  const calculateExpiry = () => {
    const currentDate = new Date();
    const expiryDate = new Date(sideBarContext.paymentPlanTrialEnd * 1000);
    const diffTime = Math.abs(expiryDate - currentDate);
    const diffDays = Math.ceil(diffTime / (1000 * 60 * 60 * 24));

    return diffDays;
  };

  const accountPlanExpiry = calculateExpiry();

  return (
    <div
      style={{
        display: "flex",
        justifyContent: "space-between",
        flexDirection: "column",
        height: "100%",
      }}
    >
      <div
        style={{
          display: "flex",
          flexDirection: "column",
          justifyContent: "space-between",
          overflowY: "hidden",
        }}
      >
        <div
          style={{
            display: "flex",
            flexDirection: "column",
            alignItems: "center",
            padding: 10,
          }}
        >
          <MyAppLogo width={200} height={60} />

          {isDevelopmentEnv() && (
            <p style={{ color: "#6878d6", fontWeight: "bold" }}>DEV</p>
          )}
        </div>

        <div style={{ overflowY: "scroll" }}>
          <Menu
            mode="inline"
            onClick={(item) => navigate(item.key)}
            theme="light"
            selectedKeys={[selectedKeys]}
            items={getFirstMenuItems()}
            openKeys={openKeys}
            onOpenChange={(openKeys) =>
              setOpenKeys(
                openKeys[openKeys.length - 1]
                  ? [openKeys[openKeys.length - 1]]
                  : []
              )
            }
          />
        </div>
      </div>

      <div>
        <Divider style={{ margin: 0 }} />

        {showPaymentPlanInfoBanner && (
          <Card
            style={{ backgroundColor: "#6878d6", margin: 8 }}
            styles={{ body: { padding: 10 } }}
          >
            <Flex justify="center" align="center">
              <Typography.Title
                level={5}
                style={{ color: "#fff", textAlign: "center" }}
              >
                {t("sideMenu.paymentPlanTrailingDaysLeft", {
                  daysLeft: accountPlanExpiry,
                  dayUnit:
                    accountPlanExpiry > 1
                      ? t("common.unit.days")
                      : t("common.unit.day"),
                })}
              </Typography.Title>
            </Flex>

            <Button
              block
              style={{ fontWeight: "bold", fontSize: 13 }}
              onClick={() => navigate(Constants.ROUTE_PATHS.PAYMENT_PLAN)}
            >
              {t("sideMenu.paymentPlanUpgradeButton")}
            </Button>
          </Card>
        )}

        <Menu
          selectable={true}
          selectedKeys={[selectedKeys]}
          mode="vertical"
          onClick={(item) => navigate(item.key)}
          items={getSecondMenuItems()}
        />
      </div>
    </div>
  );
}

/*
  const calculateExpiry = () => {
    const currentDate = new Date();
    const expiryDate = new Date(sideBarContext.accountPlanExpiry);
    const diffTime = Math.abs(expiryDate - currentDate);
    const diffDays = Math.ceil(diffTime / (1000 * 60 * 60 * 24));

    return diffDays;
  };

  const accountPlanExpiry = calculateExpiry();

{sideBarContext.accountPlanExpiry !== undefined &&
          accountPlanExpiry > 0 && (
            <Card
              style={{ backgroundColor: "#6878d6", margin: 8 }}
              bodyStyle={{ padding: 10 }}
            >
              <Flex justify="center" align="center">
                <Typography.Title
                  level={5}
                  style={{ color: "#fff", textAlign: "center" }}
                >
                  {t("sideMenu.accountPlan.infoDaysLeft", {
                    daysLeft: accountPlanExpiry,
                    dayUnit:
                      accountPlanExpiry > 1
                        ? t("common.unit.days")
                        : t("common.unit.day"),
                  })}
                </Typography.Title>
              </Flex>

              <Button
                block
                style={{ fontWeight: "bold", fontSize: 13 }}
                onClick={() => {
                  navigate(Constants.ROUTE_PATHS.PAYMENT_PLAN);
                }}
              >
                {t("sideMenu.accountPlan.buttonExtend")}
              </Button>
            </Card>
          )}
*/
