import {
  Button,
  Card,
  Col,
  Form,
  Popconfirm,
  Radio,
  Row,
  Select,
  Skeleton,
  Space,
  Switch,
  Tabs,
  Tag,
  Typography,
  notification,
} from "antd";
import {
  Constants,
  EncodeStringToBase64,
  FormatDatetime,
  handleLogout,
  myFetch,
  showInputsInvalidNotification,
  showPasswordIncorrectNotification,
  showUnkownErrorNotification,
} from "../../utils";
import { useTranslation } from "react-i18next";
import { useEffect, useRef, useState } from "react";
import {
  RequestState,
  RequestStateItem,
} from "../../Components/MyRequestStateItem";
import { EditOutlined, LogoutOutlined } from "@ant-design/icons";
import {
  MyEmailFormInput,
  MyFormInput,
  MyPasswordFormInput,
  MyUsernameFormInput,
} from "../../Components/MyFormInputs";
import { MySupsenseFallback } from "../../Components/MySupsenseFallback";
import { useSideBarContext } from "../../Contexts/SideBarContext";
import MyTable from "../../Components/MyTable";
import MyModal, {
  MyModalCloseConfirmButtonFooter,
} from "../../Components/MyModal";

const globalRequestStatePreview = {
  global: RequestState.INIT,
  yourProfile: RequestState.INIT,
  changePassword: RequestState.INIT,
  yourSessions: RequestState.INIT,
  deleteAccount: RequestState.INIT,
};

export default function UserProfile({ setUserSession }) {
  const { t } = useTranslation();
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();

  const [globalRequestState, setGlobalRequestState] = useState(
    globalRequestStatePreview
  );
  const [isRequestingLogout, setIsRequestingLogout] = useState(false);
  const [activeTab, setActiveTab] = useState(0);

  const tabItems = [
    {
      label: t("userProfile.tabs.yourProfile"),
      children: (
        <YourProfile
          notificationApi={notificationApi}
          globalRequestState={globalRequestState}
          setGlobalRequestState={setGlobalRequestState}
        />
      ),
    },
    {
      label: t("userProfile.tabs.changePassword"),
      children: (
        <ChangePassword
          notificationApi={notificationApi}
          globalRequestState={globalRequestState}
          setGlobalRequestState={setGlobalRequestState}
          setUserSession={setUserSession}
        />
      ),
    },
    {
      label: t("userProfile.tabs.yourSessions"),
      children: <YourSessions />,
    },
    {
      label: t("userProfile.tabs.yourAccount"),
      children: <YourAccount />,
    },
  ].map((item, index) => {
    return {
      key: index,
      ...item,
      children: (
        <MySupsenseFallback spinnerCentered={false}>
          {item.children}
        </MySupsenseFallback>
      ),
    };
  });

  return (
    <>
      {notificationContextHolder}

      <Tabs
        defaultActiveKey={0}
        items={tabItems}
        activeKey={activeTab}
        onChange={(key) => setActiveTab(key)}
        tabBarExtraContent={
          <Space>
            <RequestStateItem
              state={globalRequestState.global}
              setRequestState={(state) =>
                setGlobalRequestState({
                  ...globalRequestState,
                  global: state,
                })
              }
            />

            <Button
              type="primary"
              loading={isRequestingLogout}
              icon={<LogoutOutlined />}
              onClick={() => {
                setIsRequestingLogout(true);

                myFetch({
                  url: "/user/auth/logout",
                  method: "DELETE",
                  notificationApi: notificationApi,
                  t: t,
                })
                  .then(() =>
                    handleLogout({
                      setUserSession: setUserSession,
                    })
                  )
                  .catch(() => setIsRequestingLogout(false));
              }}
            >
              {t("common.button.logout")}
            </Button>
          </Space>
        }
      />
    </>
  );
}

function YourProfile({
  notificationApi,
  globalRequestState,
  setGlobalRequestState,
}) {
  const { t, i18n } = useTranslation();
  const [form] = Form.useForm();

  const sideBarContext = useSideBarContext();

  const delayTimeout = useRef();
  const [requestData, setRequestData] = useState({});

  const language = Form.useWatch("language", form);
  const analyticsEnabled = Form.useWatch("analyticsEnabled", form);
  const username = Form.useWatch("username", form);
  const email = Form.useWatch("email", form);

  const requestState = globalRequestState.yourProfile;

  useEffect(() => {
    myFetch({
      url: "/user/profile/settings",
      method: "GET",
      notificationApi: notificationApi,
      t: t,
    })
      .then((data) => {
        setRequestData(data);

        form.setFieldsValue({
          language: data.language,
          analyticsEnabled: data.analytics_enabled,
          username: data.username,
          email: data.email,
        });
      })
      .catch(() => {
        setGlobalRequestState({
          ...globalRequestState,
          global: RequestState.FAILED,
        });
      });
  }, []);

  useEffect(() => {
    if (
      language === undefined ||
      analyticsEnabled === undefined ||
      username === undefined
    )
      return;

    if (RequestState.INIT === requestState) {
      setGlobalRequestState({
        ...globalRequestState,
        global: RequestState.NOTHING,
        yourProfile: RequestState.NOTHING,
      });
      return;
    }

    setGlobalRequestState({
      ...globalRequestState,
      global: RequestState.REQUESTING,
    });

    if (delayTimeout.current) {
      clearTimeout(delayTimeout.current);
    }

    delayTimeout.current = setTimeout(() => {
      form
        .validateFields()
        .then(() => {
          let body = {};

          if (language !== requestData.language) {
            body.language = language;
          }

          if (analyticsEnabled !== requestData.analytics_enabled) {
            body.analyticsEnabled = analyticsEnabled;
          }

          if (username !== requestData.username) {
            body.username = username;
          }

          if (Object.keys(body).length === 0) {
            setGlobalRequestState({
              ...globalRequestState,
              global: RequestState.NOTHING,
            });
            return;
          }

          myFetch({
            url: "/user/profile/settings",
            method: "POST",
            body: body,
            notificationApi: notificationApi,
            t: t,
          })
            .then(() => {
              setGlobalRequestState({
                ...globalRequestState,
                global: RequestState.SUCCESS,
              });

              // Update requestData to the new values
              setRequestData({
                ...requestData,
                language: language,
                analytics_enabled: analyticsEnabled,
                username: username,
              });

              sideBarContext.setUsername(username);
            })
            .catch(() => {
              setGlobalRequestState({
                ...globalRequestState,
                global: RequestState.FAILED,
              });
            });
        })
        .catch(() => {
          showInputsInvalidNotification(notificationApi, t);

          setGlobalRequestState({
            ...globalRequestState,
            global: RequestState.NOTHING,
          });
        });
    }, 500);
  }, [language, analyticsEnabled, username]);

  return (
    <Card title={t("userProfile.yourProfile.cardTitle")}>
      <Form form={form} layout="vertical" requiredMark={false}>
        <Form.Item
          name="language"
          label={t("userProfile.yourProfile.language")}
        >
          {requestState === RequestState.INIT ? (
            <Skeleton.Input active block></Skeleton.Input>
          ) : (
            <Select
              style={{ width: "100%" }}
              options={[
                {
                  value: "en",
                  label: "English",
                },
                {
                  value: "de",
                  label: "Deutsch",
                },
              ]}
              onChange={(e) => i18n.changeLanguage(e)}
            />
          )}
        </Form.Item>

        <Form.Item
          name="analyticsEnabled"
          valuePropName="checked"
          label={t("userProfile.yourProfile.analytics")}
          extra={t("userProfile.yourProfile.analyticsDescription")}
        >
          {requestState === RequestState.INIT ? (
            <Skeleton.Button shape="round" active />
          ) : (
            <Switch>{t("userProfile.yourProfile.analytics")}</Switch>
          )}
        </Form.Item>

        <MyUsernameFormInput
          showSkeleton={requestState === RequestState.INIT}
        />

        <MyFormInput
          formItemName="email"
          label={t("common.email")}
          propsInput={{
            disabled: true,
            value: email,
          }}
          showSkeleton={requestState === RequestState.INIT}
          InputFatherElement={(props) => (
            <Space.Compact block>
              {props.children}
              <YourProfileChangeEmailModal />
            </Space.Compact>
          )}
        />
      </Form>
    </Card>
  );
}

function YourProfileChangeEmailModal() {
  const { t } = useTranslation();

  const [form] = Form.useForm();
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();
  const [isModalOpen, setIsModalOpen] = useState(false);

  const [isRequesting, setIsRequesting] = useState(false);

  const handleModalClose = () => setIsModalOpen(false);

  useEffect(() => {
    if (!isModalOpen) return;

    form.resetFields();
  }, [isModalOpen]);

  return (
    <>
      {notificationContextHolder}

      <Button
        type="primary"
        icon={<EditOutlined />}
        onClick={() => setIsModalOpen(true)}
      />

      <MyModal
        title={t("userProfile.changeEmailModal.title")}
        isOpen={isModalOpen}
        onCancel={handleModalClose}
        footer={
          <MyModalCloseConfirmButtonFooter
            onCancel={handleModalClose}
            isConfirmButtonLoading={isRequesting}
            onConfirm={() => {
              form
                .validateFields()
                .then((values) => {
                  setIsRequesting(true);

                  myFetch({
                    url: "/user/profile/email",
                    method: "POST",
                    body: {
                      email: values.email,
                      password: EncodeStringToBase64(values.password),
                    },
                  })
                    .then(() => {
                      setIsRequesting(false);
                      setIsModalOpen(false);

                      notificationApi["info"]({
                        message: t(
                          "userProfile.changeEmailModal.request.200.title"
                        ),
                        description: t(
                          "userProfile.changeEmailModal.request.200.description"
                        ),
                      });
                    })
                    .catch(() => {
                      setIsRequesting(false);
                      showPasswordIncorrectNotification(notificationApi, t);
                    });
                })
                .catch(() => setIsRequesting(false));
            }}
          />
        }
      >
        <Form form={form} layout="vertical" requiredMark={false}>
          <Typography.Paragraph type="secondary">
            {t("userProfile.changeEmailModal.info")}
          </Typography.Paragraph>

          <MyEmailFormInput hasFeedback newEmail />

          <MyPasswordFormInput />
        </Form>
      </MyModal>
    </>
  );
}

function ChangePassword({
  notificationApi,
  globalRequestState,
  setGlobalRequestState,
  setUserSession,
}) {
  const { t } = useTranslation();

  const [form] = Form.useForm();

  const handlePasswordChange = () => {
    form
      .validateFields()
      .then((values) => {
        console.log(values);

        setGlobalRequestState({
          ...globalRequestState,
          global: RequestState.REQUESTING,
          changePassword: RequestState.REQUESTING,
        });

        myFetch({
          url: "/user/profile/password",
          method: "POST",
          body: {
            currentPassword: EncodeStringToBase64(values.currentPassword),
            newPassword: EncodeStringToBase64(values.newPassword),
          },
          notificationApi: notificationApi,
          t: t,
        })
          .then(() => {
            form.resetFields();

            setGlobalRequestState({
              ...globalRequestState,
              global: RequestState.SUCCESS,
              changePassword: RequestState.NOTHING,
            });

            handleLogout({
              setUserSession: setUserSession,
            });
          })
          .catch((err) => {
            setGlobalRequestState({
              ...globalRequestState,
              global: RequestState.FAILED,
              changePassword: RequestState.NOTHING,
            });

            if (err === 400) {
              showPasswordIncorrectNotification(notificationApi, t);
            }
          });
      })
      .catch(() => showInputsInvalidNotification(notificationApi, t));
  };

  return (
    <Card title={t("userProfile.changePassword.cardTitle")}>
      <Form
        form={form}
        layout="vertical"
        autoComplete="off"
        requiredMark={false}
      >
        <MyPasswordFormInput
          propsInput={{
            id: "currentPassword",
          }}
          formItemName="currentPassword"
          label={t("userProfile.changePassword.currentPassword")}
          inputPlaceholder={t(
            "userProfile.changePassword.currentPasswordPlaceholder"
          )}
        />

        <MyPasswordFormInput
          propsInput={{
            id: "newPassword",
          }}
          formItemName="newPassword"
          label={t("userProfile.changePassword.newPassword")}
          inputPlaceholder={t(
            "userProfile.changePassword.newPasswordPlaceholder"
          )}
        />

        <MyPasswordFormInput
          propsInput={{
            id: "confirmNewPassword",
          }}
          formItemName="confirmNewPassword"
          label={t("userProfile.changePassword.confirmNewPassword")}
          inputPlaceholder={t(
            "userProfile.changePassword.confirmNewPasswordPlaceholder"
          )}
          propsFormItem={{
            dependencies: ["newPassword"],
          }}
          formItemRules={({ getFieldValue }) => ({
            validator(_, value) {
              if (!value || getFieldValue("newPassword") === value) {
                return Promise.resolve();
              }
              return Promise.reject(
                new Error(
                  t("userProfile.changePassword.confirmNewPasswordMismatch")
                )
              );
            },
          })}
        />

        <Form.Item extra={t("userProfile.changePassword.buttonInfo")}>
          <Space>
            <Button
              type="primary"
              onClick={handlePasswordChange}
              loading={
                globalRequestState.changePassword === RequestState.REQUESTING
              }
            >
              {t("userProfile.changePassword.button")}
            </Button>
          </Space>
        </Form.Item>
      </Form>
    </Card>
  );
}

function YourSessions() {
  const { t } = useTranslation();

  const [isRequesting, setIsRequesting] = useState(true);
  const [requestData, setRequestData] = useState({
    sessions: [],
    currentSession: "",
  });

  const getTableColumns = () => {
    return [
      {
        title: t("userProfile.yourSessions.tableColumns.device"),
        dataIndex: "device",
        key: "device",
      },
      {
        title: t("userProfile.yourSessions.tableColumns.browser"),
        dataIndex: "browser",
        key: "browser",
      },
      {
        title: t("userProfile.yourSessions.tableColumns.lastUsed"),
        dataIndex: "last_used",
        key: "last_used",
      },
      {
        title: t("common.action"),
        dataIndex: "actions",
        key: "actions",
      },
    ];
  };

  const getTableItems = () => {
    return requestData.sessions.map((item) => {
      return {
        key: item.id,
        device: item.os,
        browser: item.browser,
        last_used: (
          <Space>
            {FormatDatetime(item.last_used)}
            {requestData.currentSession === item.id && (
              <Tag color="green">
                {t("userProfile.yourSessions.tagCurrentSession")}
              </Tag>
            )}
          </Space>
        ),
        actions: (
          <Popconfirm
            title={t("userProfile.yourSessions.popconfirm.title")}
            description={t("userProfile.yourSessions.popconfirm.description")}
            onConfirm={() => {
              setIsRequesting(true);

              myFetch({
                url: `/user/profile/sessions/${item.id}`,
                method: "DELETE",
              })
                .then(() => {
                  setIsRequesting(false);

                  // If the current session is deleted, reload the page
                  if (item.id === requestData.currentSession) {
                    window.location.reload();
                  }

                  requestSessions();
                })
                .catch(() => setIsRequesting(false));
            }}
            okText={t("common.button.logout")}
            cancelText={t("common.button.cancel")}
          >
            <Button type="link">{t("common.button.delete")}</Button>
          </Popconfirm>
        ),
      };
    });
  };

  const requestSessions = () => {
    setIsRequesting(true);

    myFetch({
      url: "/user/profile/sessions",
      method: "GET",
    })
      .then((res) => {
        setIsRequesting(false);
        setRequestData(res);
      })
      .catch(() => setIsRequesting(false));
  };

  useEffect(() => requestSessions(), []);

  return (
    <Card title={t("userProfile.yourSessions.cardTitle")}>
      <MyTable
        props={{
          loading: isRequesting,
          columns: getTableColumns(),
          dataSource: getTableItems(),
        }}
      />
    </Card>
  );
}

function YourAccount() {
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();

  return (
    <>
      {notificationContextHolder}

      <Row gutter={[16, 16]}>
        <Col span={24}>
          <YourAccountExportDataCard notificationApi={notificationApi} />
        </Col>

        <Col span={24}>
          <YourAccountDeleteAccountCard notificationApi={notificationApi} />
        </Col>
      </Row>
    </>
  );
}

function YourAccountExportDataCard({ notificationApi }) {
  const { t } = useTranslation();

  const [form] = Form.useForm();

  const [isModalOpen, setIsModalOpen] = useState(false);
  const [isRequesting, setIsRequesting] = useState(false);

  const handleModalClose = () => setIsModalOpen(false);

  useEffect(() => {
    if (!isModalOpen) return;

    form.resetFields();
  }, [isModalOpen]);

  return (
    <Card title={t("userProfile.accountExport.cardTitle")}>
      <Space direction="vertical">
        <Typography.Text>{t("userProfile.accountExport.info")}</Typography.Text>

        <Button type="default" onClick={() => setIsModalOpen(true)}>
          {t("userProfile.accountExport.button")}
        </Button>
      </Space>

      <MyModal
        title={t("userProfile.accountExport.modal.title")}
        isOpen={isModalOpen}
        onCancel={handleModalClose}
        footer={
          <MyModalCloseConfirmButtonFooter
            isConfirmButtonLoading={isRequesting}
            onConfirm={() => {
              form
                .validateFields()
                .then((values) => {
                  setIsRequesting(true);

                  myFetch({
                    url: "/user/profile/export",
                    method: "POST",
                    body: {
                      email: values.email,
                      password: EncodeStringToBase64(values.password),
                    },
                  })
                    .then(() => {
                      setIsRequesting(false);
                      setIsModalOpen(false);

                      notificationApi["info"]({
                        message: t(
                          "userProfile.accountExport.modal.request.200.title"
                        ),
                        description: t(
                          "userProfile.accountExport.modal.request.200.description"
                        ),
                      });
                    })
                    .catch((errStatus) => {
                      setIsRequesting(false);

                      if (errStatus === 400) {
                        showPasswordIncorrectNotification(notificationApi, t);
                        return;
                      }

                      showUnkownErrorNotification(notificationApi, t);
                    });
                })
                .catch(() => showInputsInvalidNotification(notificationApi, t));
            }}
            onCancel={handleModalClose}
          />
        }
      >
        <Form form={form} layout="vertical" requiredMark={false}>
          <Typography.Paragraph type="secondary">
            {t("userProfile.accountExport.modal.info")}
          </Typography.Paragraph>

          <MyEmailFormInput disableEmailCheck />

          <MyPasswordFormInput />
        </Form>
      </MyModal>
    </Card>
  );
}

function YourAccountDeleteAccountCard({ notificationApi }) {
  const { t } = useTranslation();

  const [form] = Form.useForm();

  const [isModalOpen, setIsModalOpen] = useState(false);
  const [isRequesting, setIsRequesting] = useState(false);

  const handleModalClose = () => setIsModalOpen(false);
  /*useEffect(() => {
    if (!isModalOpen) return;

    form.resetFields();
  }, [isModalOpen]); */

  return (
    <Card title={t("userProfile.deleteAccount.cardTitle")}>
      <Space direction="vertical">
        <Typography.Text>{t("userProfile.deleteAccount.info")}</Typography.Text>

        <Button type="primary" danger onClick={() => setIsModalOpen(true)}>
          {t("userProfile.deleteAccount.button")}
        </Button>
      </Space>

      <MyModal
        title={t("userProfile.deleteAccount.modal.title")}
        isOpen={isModalOpen}
        onCancel={handleModalClose}
        footer={
          <MyModalCloseConfirmButtonFooter
            danger
            isConfirmButtonLoading={isRequesting}
            onConfirm={() => {
              form
                .validateFields()
                .then((values) => {
                  setIsRequesting(true);

                  myFetch({
                    url: "/user/profile/delete",
                    method: "DELETE",
                    body: {
                      feedback: values.improve,
                      reason: values["deleteReason"],
                      password: EncodeStringToBase64(values.password),
                    },
                  })
                    .then(() => {
                      setIsRequesting(false);

                      window.location.reload();
                    })
                    .catch((errStatus) => {
                      setIsRequesting(false);

                      if (errStatus === 400) {
                        showPasswordIncorrectNotification(notificationApi, t);
                      }
                    });
                })
                .catch(() => showInputsInvalidNotification(notificationApi, t));
            }}
            onCancel={handleModalClose}
          />
        }
      >
        <Form form={form} layout="vertical" requiredMark={false}>
          <Typography.Paragraph type="secondary">
            {t("userProfile.deleteAccount.modal.info")}
          </Typography.Paragraph>

          <Form.Item
            name="deleteReason"
            label={t("userProfile.deleteAccount.modal.why")}
            required
            rules={[
              {
                required: true,
                message: t("userProfile.deleteAccount.modal.whyRuleRequired"),
              },
            ]}
          >
            <Radio.Group>
              <Space direction="vertical">
                <Radio value="notNeeded">
                  {t("userProfile.deleteAccount.modal.radioOptions.notNeeded")}
                </Radio>
                <Radio value="notUseful">
                  {t("userProfile.deleteAccount.modal.radioOptions.notUseful")}
                </Radio>
                <Radio value="other">
                  {t("userProfile.deleteAccount.modal.radioOptions.other")}
                </Radio>
              </Space>
            </Radio.Group>
          </Form.Item>

          <MyFormInput
            formItemName="improve"
            minLength={Constants.GLOBALS.MIN_FEEDBACK_LENGTH}
            maxLength={Constants.GLOBALS.MAX_FEEDBACK_LENGTH}
            inputType="textarea"
            propsInput={{
              showCount: true,
            }}
            label={t("userProfile.deleteAccount.modal.improveLabel")}
            ruleMessageValueRequired={t(
              "userProfile.deleteAccount.modal.improveRequired"
            )}
            ruleMessageValueMinLengthRequired={t(
              "userProfile.deleteAccount.modal.improveMinLengthRequired",
              {
                minLength: Constants.GLOBALS.MIN_FEEDBACK_LENGTH,
              }
            )}
          />

          <MyPasswordFormInput />
        </Form>
      </MyModal>
    </Card>
  );
}
